/* $Id: unit.h 414 2009-10-25 14:36:18Z truebrain $ */

#ifndef UNIT_H
#define UNIT_H

/**
 * Types of Units available in the game.
 */
typedef enum UnitType {
	UNIT_CARRYALL        = 0,
	UNIT_ORNITHOPTER     = 1,
	UNIT_INFANTRY        = 2,
	UNIT_TROOPERS        = 3,
	UNIT_SOLDIER         = 4,
	UNIT_TROOPER         = 5,
	UNIT_SABOTEUR        = 6,
	UNIT_LAUNCHER        = 7,
	UNIT_DEVIATOR        = 8,
	UNIT_TANK            = 9,
	UNIT_SIEGE_TANK      = 10,
	UNIT_DEVASTATOR      = 11,
	UNIT_SONIC_TANK      = 12,
	UNIT_TRIKE           = 13,
	UNIT_RAIDER_TRIKE    = 14,
	UNIT_QUAD            = 15,
	UNIT_HARVESTER       = 16,
	UNIT_MGV             = 17,
	UNIT_MISSLE_HOUSE    = 18,
	UNIT_MISSLE_ROCKET   = 19,
	UNIT_MISSLE_TURRET   = 20,
	UNIT_MISSLE_DEVIATOR = 21,
	UNIT_MISSLE_TROOPER  = 22,
	UNIT_BULLET          = 23,
	UNIT_SONIC_BLAST     = 24,
	UNIT_SANDWORM        = 25,
	UNIT_FRIGATE         = 26,

	UNIT_MAX             = 27,
	UNIT_INVALID         = 0xFFFF
} UnitType;

MSVC_PACKED_BEGIN
/**
 * A Unit as stored in the memory.
 */
typedef struct Unit {
	/* 0000(2)   */ PACK uint16 index;                      /*!< The index of the Unit in the array. */
	/* 0002(1)   */ PACK uint8  type;                       /*!< Type of Unit. */
	/* 0003(1)   */ PACK uint8  linkedBuildingID;           /*!< Building we are linked to, or 0xFF if we are not linked to a building. */
	/* 0004(2)   */ PACK uint16 flags;                      /*!< ?? Bitflags. 0x0001 - Used, 0x0002 - Allocated?, 0x0004 - Being-built?, 0x0200 - Scenario Unit. */
	/* 0006(2)   */ PACK uint16 variable_06;                /*!< ?? */
	/* 0008(1)   */ PACK uint8  houseID;                    /*!< House of Unit. */
	/* 0009(1)   */ PACK uint8  variable_09;                /*!< ?? */
	/* 000A(4)   */ PACK tile32 position;                   /*!< Position on the map. */
	/* 000E(2)   */ PACK uint16 hitpoints;                  /*!< Current hitpoints left. */
	/* 0010(2)   */ PACK uint16 variable_10;                /*!< ?? */
	/* 0012()    */ PACK uint8   unknown_0012[0x0037];
	/* 0049(4)   */ PACK csip32 variable_49;                /*!< ?? */
	/* 004D(2)   */ PACK uint16 variable_4D;                /*!< ?? */
	/* 004F(1)   */ PACK uint8  actionID;                   /*!< Current action. */
	/* 0050(1)   */ PACK uint8  variable_50;                /*!< ?? */
	/* 0051(1)   */ PACK uint8  variable_51;                /*!< ?? */
	/* 0052(2)   */ PACK uint16 variable_52;                /*!< ?? */
	/* 0054(2)   */ PACK uint8   unknown_0054[0x0002];
	/* 0056(2)   */ PACK uint16 variable_56;                /*!< ?? */
	/* 0058(1)   */ PACK uint8  sandwormLeft;               /*!< How much units the sandworm will eat before it disappears. */
	/* 0059(1)   */ PACK uint8  deviated;                   /*!< ?? If non-zero, the unit is deviated, but what does it hold exactly? */
	/* 005A(4)   */ PACK tile32 variable_5A;                /*!< ?? */
	/* 005E(4)   */ PACK tile32 variable_5E;                /*!< ?? */
	/* 0062()    */ PACK uint8   unknown_0062[0x0002];
	/* 0064(1)   */ PACK uint8  variable_64;                /*!< ?? */
	/* 0065()    */ PACK uint8   unknown_0066[0x0007];
	/* 006C(1)   */ PACK uint8  variable_6C;                /*!< ?? */
	/* 006D(1)   */ PACK uint8  variable_6D;                /*!< ?? */
	/* 006E(1)   */ PACK uint8  variable_6E;                /*!< ?? */
	/* 006F(1)   */ PACK uint8  variable_6F;                /*!< ?? */
	/* 0070(1)   */ PACK uint16 variable_70;                /*!< ?? */
	/* 0072(1)   */ PACK uint8  variable_72;                /*!< ?? */
	/* 0073()    */ PACK uint8   unknown_0073[0x000D];
} GCC_PACKED Unit;
MSVC_PACKED_END
assert_compile(sizeof(Unit) == 0x80);

MSVC_PACKED_BEGIN
/**
 * Static information per Unit type.
 */
typedef struct UnitInfo {
	/* 0000()    */ PACK uint8   unknown_0000[0x0010];
	/* 0010(2)   */ PACK uint16 hitpoints;                  /*!< Default hitpoints for this Unit. */
	/* 0012()    */ PACK uint8   unknown_0012[0x0006];
	/* 0018(2)   */ PACK uint16 buildTime;                  /*!< Time required to build this Unit. */
	/* 001A()    */ PACK uint8   unknown_001A[0x000E];
	/* 0028(2)   */ PACK uint16 variable_28;                /*!< ?? */
	/* 002A()    */ PACK uint8   unknown_002A[0x0008];
	/* 0032(2)   */ PACK uint16 indexStart;                 /*!< At Unit create, between this and indexEnd (including) a free index is picked. */
	/* 0034(2)   */ PACK uint16 indexEnd;                   /*!< At Unit create, between indexStart and this (including) a free index is picked. */
	/* 0036()    */ PACK uint8   unknown_0036[0x0006];
	/* 003C(2)   */ PACK uint16 variable_3C;                /*!< ?? */
	/* 003E()    */ PACK uint8   unknown_003E[0x000A];
	/* 0048(2)   */ PACK uint16 variable_48;                /*!< ?? */
	/* 004A()    */ PACK uint8   unknown_004A[0x0010];
} GCC_PACKED UnitInfo;
MSVC_PACKED_END
assert_compile(sizeof(UnitInfo) == 0x5A);

extern UnitInfo *g_unitInfo;

uint8 Unit_GetHouseID(Unit *u);
extern struct Unit *Unit_Create(uint16 index, uint8 typeID, uint8 houseID, tile32 position, uint16 var10);


extern void emu_Unit_GetHouseID();
extern void emu_Unit_Create();

#endif /* UNIT_H */
